import buw.*;

/**
 * example 5: querying a web service
 * using: VBox, ListView, Separator, Label, TextButton & Input
 * nothing new with BUW here, it is only a demonstration of its interoperability with haxe.Http class
 */

typedef Result = {
	var type : String;
	var display_name : String;
	var lat : Float;
	var lon : Float;
}

class Example5 {
	public static function main() {
		new Example5();
	}

	static inline var WSURI : String = "https://nominatim.openstreetmap.org/search?format=json&q=";
	var search : Input;
	var resultsList : ListView<Result>;

	function new() {
		var rootWindow = new VBox();
		rootWindow.pack(new Label("Enter search location:"));
		search = new Input(40);
		rootWindow.pack(search);
		rootWindow.pack(new TextButton(onSearchClick, "Search"));
		Screen.display(rootWindow);
		rootWindow.pack(new Separator());
		resultsList = new ListView(renderSearch, onResultClick);
		rootWindow.pack(resultsList);
	}

	function onSearchClick(w : TextButton) {
		function handleResponse(jsonData : String) {
			var results : Array<Result> = Lambda.filter(haxe.Json.parse(jsonData),
											function (r: Result) { return r.type == "city"; });
			Lambda.iter(results, function (r : Result) {
				var parts : Array<String> = r.display_name.split(", ");
				r.display_name = parts[0] + ", " + parts[2];
			});
			resultsList.source = results;
		}

		if (search.value != "") {
			var httpRequest = new haxe.Http(WSURI + search.value);
			httpRequest.onError = function (msg : String) {
				Sys.println(msg);
			}
			httpRequest.addHeader("User-Agent", "haxe.tuxfamily.org");

			//~ httpRequest.onData = handleResponse; 
			//~ httpRequest.request(); //invalid X509 certificate (issued by Let's Encrypt)

			var bytesOutput = new haxe.io.BytesOutput();
			var socket = new sys.ssl.Socket();
			socket.verifyCert = false;
			httpRequest.customRequest(false, bytesOutput, socket, "GET");
			handleResponse(bytesOutput.getBytes().toString());
		}
	}

	function renderSearch (r : Result) : Widget {
		return new Label(r.display_name);
	}

	function onResultClick(r : Result) {
		Sys.println(r.type);
	}
}
